#!/bin/sh

########################################################################################
# Description:	Backup-Script for the vip1710
#		Makes a backup of the config-files
# Author:	Thomas Barth
# Date:		01.09.2011
# Version:	1.00
#
# Hint:		Feel free to modify the script
#		Feel also free to contact me
#		nessie_1980@hotmail.com
########################################################################################

DESCRIPTION="# This script makes a backup of specified files of your USB-stick or NFS-mount.
# In the first step the script checks if all directories available. If they aren\`t available the script tries to create them.
# After that step the script backups the files which were specified in the files \"COPY_CONFIGFILE\" and \"COMPARE_CONFIGFILE\". 
# The files \"COPY_CONFIGFILE\" contains the files which will be backuped to the specified directory
# and later restored by the restore.sh-script to your USB-Stick. (1:1-Copy per CP-command)
# The files \"COMPARE_CONFIGFILE\" contains the files which will be backuped to the specified directory.
# When you will restore that files the restore.sh-script compare the content of the back-file(s) with the new file(s) on
# the USB-stick or NFS-mount and restores your values of the parameters.\n"

### variables ###
SRC="/mnt/"						# directory of the destination (USB-Stick)	(changeable)
DEST="/mnt/backup/"					# directory for the backup			(changeable)
COMPARE=$DEST"compare/"					# directory for the compare-files		(unchangeable)
COPY=$DEST"copy/"					# directory for the copy-files			(unchangeable)
LOG=$DEST"log/"						# directory for the log-file			(unchangeable)
SCRIPTPATH=$(cd -P "$( dirname "$0" )" && pwd )"/"	# directory of the script			(unchangeable)
COPY_CONFIGFILE=$SCRIPTPATH"copy.conf"			# copy-configfile				(unchangeable)
COMPARE_CONFIGFILE=$SCRIPTPATH"compare.conf"		# compare-configfile				(unchangeable)
TYPE="backup"						# backup					(unchangeable)
FILENAME=""						# name of the file which has been copied	(unchangeable)
DATELOGFILE=`date +"%d_%b_%Y_%H_%M_%S"`			#						(changeable)
LOGFILE=$LOG$TYPE"_"$DATELOGFILE".log"			# logfile					(unchangeable)
DATE=`date +"%d.%b.%Y at %H:%M:%S"`			#						(changeable)
MAXLOGFILES=5						# assign how many logfiles stored		(changeable)

# (unchangeable)
ERROR_COUNTER=0			# counter-variable
LOG_COUNTER=0			# counter-variable

########################################################################################
### functions									     ###
########################################################################################
### create log-file ###
### function_createlog
function_createlog ()
{
 echo "### Creating log-file ###"
 if [ -d "$LOG" ]
  then
     echo "   Directory \"$LOG\" exists..."
     echo ""
  else
     echo "   Directory \"$LOG\" doesn\`t exist!"
     mkdir -p $LOG 2> /dev/null
     if [ -d "$LOG" ]
       then
          echo "   Directory \"$LOG\" has been created..."
	  echo ""
       else
          echo "   Directory \"$LOG\" couldn\`t be created! Abort script!"
          echo ""
          exit 0
     fi
 fi
 touch $LOGFILE 2> /dev/null
 if [ "$?" != "0" ]
  then
     echo "   Log-file can´t be created! Abort script!"
     echo ""
     exit 0
 fi

 echo "### Log-file form "$DATE" ###" > $LOGFILE
 echo "   $LOGFILE"
 echo "   $LOGFILE" >> $LOGFILE
 echo ""
 echo "" >> $LOGFILE
}

### test if all directories are avialable ###
### function_directory directories (which should be checked)
function_directory ()
{
 for DIR in $@
  do
    echo "### Test if the directory \"$DIR\" exists ###" 
    echo "### Test if the directory \"$DIR\" exists ###" >> $LOGFILE
    if [ -d "$DIR" ]
     then
        echo "   Directory \"$DIR\" exists..."
        echo "   Directory \"$DIR\" exists..." >> $LOGFILE
        echo ""
        echo "" >> $LOGFILE
     else
        if [ "$DIR" != "$SRC" ]
         then
            echo "   Directory \"$DIR\" doesn\`t exist!"
            echo "   Directory \"$DIR\" doesn\`t exist!" >> $LOGFILE
	    mkdir -p $DIR 2> /dev/null
            if [ -d "$DIR" ]
             then
                echo "   Directory \"$DIR\" has been created..."
                echo "   Directory \"$DIR\" has been created..." >> $LOGFILE
	        echo ""
	        echo "" >> $LOGFILE
             else
                echo "   Directory \"$DIR\" couldn\`t be created! Abort script!"
                echo "   Directory \"$DIR\" couldn\`t be created! Abort script!" >> $LOGFILE
 	        echo ""
 	        echo "" >> $LOGFILE
                exit 0
            fi
        else
           echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!"
           echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
           exit 0
        fi
    fi
 done
}

### test if the copy.conf-file exists ###
### function_fileexists file
function_fileexists ()
{
 echo "### Test if the file \"$1\" exists ###" 
 echo "### Test if the file \"$1\" exists ###" >> $LOGFILE
 if [ -f "$1" ]
   then
      echo "   File \"$1\" exists..."
      echo "   File \"$1\" exists..." >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
   else
      echo "   File \"$1\" doesn\`t exists! Abort script!"
      echo "   File \"$1\" doesn\`t exists! Abort script!" >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
      exit 0
 fi
}

### copy files or directories from config-file or directory ###
### function_copy config-file source destination
function_copy ()
{
 if [ "$4" = "backup" ]
  then
     ### erase the content of the directory before the backup starts###
     echo "### Erase old backup-files or directories ###"
     echo "### Erase old backup-files or directories ###" >> $LOGFILE

     for FILES in `ls $3`
      do
        echo "   Erase: rm -r $3$FILES"
	echo "   Erase: rm -r $3$FILES" >> $LOGFILE
    	rm -r $3$FILES 2> /dev/null
    	if [ ! -e $3$FILES ]
     	 then
            echo "   File or directory \"$3$FILES\" was erased..."
            echo "   File or directory \"$3$FILES\" was erased..." >> $LOGFILE
	    echo ""
	    echo "" >> $LOGFILE
         else
	    echo "   -=>FAILURE<=-"
	    echo "   -=>FAILURE<=-" >> $LOGFILE
            echo "   File or directory \"$3$FILES\" wasn\`t erased!"
            echo "   File or directory \"$3$FILES\" wasn\`t erased!" >> $LOGFILE
            echo ""
            echo "" >> $LOGFILE
	    ERROR_COUNTER=$((ERROR_COUNTER + 1))
       fi
     done
 fi

 echo "### Copy new files or directories ###"
 echo "### Copy new files or directories ###" >> $LOGFILE

 ### determine the files or directoies which should be backuped ###
 for FILES in `sed -e '/^[  ]*#/d' -e '/^$/d' $1`
  do
    ### determine the name of the file or directory which was copied ###
    OIFS=$IFS
    IFS="/"
    for I in $FILES
     do
      FILENAME=$I
    done
    IFS=$OIFS

    ### test if the file or directory exists ###
    if [ "$4" = "backup" ]
     then
        if [ -f "$2$FILES" -o -d "$2$FILES" ]
         then
            echo "   File or directory found: \"$2$FILES\""
            echo "   File or directory found: \"$2$FILES\"" >> $LOGFILE
	    cp -R $2$FILES $3$FILENAME 2> /dev/null

	    ### test if the copy was successful ###
	    if [ -f "$3$FILENAME" -o -d "$3$FILENAME" ]
             then
                echo "   File or directory \"$2$FILES\" copied to \"$3$FILENAME\"..."
                echo "   File or directory \"$2$FILES\" copied to \"$3$FILENAME\"..." >> $LOGFILE
             else
	        echo "   -=>FAILURE<=-"
	        echo "   -=>FAILURE<=-" >> $LOGFILE
                echo "   File or directory \"$2$FILES\" wasn\`t copied to \"$3$FILENAME\"!"
                echo "   File or directory \"$2$FILES\" wasn\`t copied to \"$3$FILENAME\"!" >> $LOGFILE
	        ERROR_COUNTER=$((ERROR_COUNTER+1))
           fi
           echo ""
           echo "" >> $LOGFILE
        else
	   echo "   -=>FAILURE<=-"
	   echo "   -=>FAILURE<=-" >> $LOGFILE
           echo "   File or directory \"$2$FILES\" not found!"
           echo "   File or directory \"$2$FILES\" not found!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
           ERROR_COUNTER=$((ERROR_COUNTER+1))
        fi
    elif [ "$4" = "restore" ]
     then
        if [ -f "$2$FILENAME" -o -d "$2$FILENAME" ]
         then
            echo "   File or directory found: \"$2$FILENAME\""
            echo "   File or directory found: \"$2$FILENAME\"" >> $LOGFILE
	    cp -R $2$FILENAME $3$FILES 2> /dev/null

	    ### test if the copy was successful ###
	    if [ -f "$3$FILES" -o -d "$3$FILES" ]
             then
                echo "   File or directory \"$2$FILENAME\" copied to \"$3$FILES\"..."
                echo "   File or directory \"$2$FILENAME\" copied to \"$3$FILES\"..." >> $LOGFILE
             else
	        echo "   -=>FAILURE<=-"
	        echo "   -=>FAILURE<=-" >> $LOGFILE
                echo "   File or directory \"$2$FILENAME\" wasn\`t copied to \"$3$FILES\"!"
                echo "   File or directory \"$2$FILENAME\" wasn\`t copied to \"$3$FILES\"!" >> $LOGFILE
	        ERROR_COUNTER=$((ERROR_COUNTER+1))
            fi
            echo ""
            echo "" >> $LOGFILE
         else
	    echo "   -=>FAILURE<=-"
	    echo "   -=>FAILURE<=-" >> $LOGFILE
            echo "   File or directory \"$2$FILENAME\" not found!"
            echo "   File or directory \"$2$FILENAME\" not found!" >> $LOGFILE
            echo ""
            echo "" >> $LOGFILE
            ERROR_COUNTER=$((ERROR_COUNTER+1))
        fi
    fi
 done
}

### cleanup log-directory ###
### function_clean search-string (in the filename)
function_clean ()
{
 for FILES in `ls -r $LOG | grep $1`
  do
   if [ $LOG_COUNTER -gt $(( MAXLOGFILES - 1 )) ]
    then
       echo "   Log-file: \"$FILES\" will be deleted!"
       echo "   Log-file: \"$FILES\" will be deleted!" >> $LOGFILE
       rm $LOG$FILES 2> /dev/null
       if [ -f "$LOG$FILEs" ]
        then
	   echo "   -=>FAILURE<=-"
	   echo "   -=>FAILURE<=-" >> $LOGFILE
           echo "   Log-file: \"$FILES\" wasn\`t deleted!"
           echo "   Log-file: \"$FILES\" wasn\`t deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
	   ERROR_COUNTER=$(( ERROR_COUNTER + 1 ))
        else
           echo "   Log-file: \"$FILES\" was deleted!"
           echo "   Log-file: \"$FILES\" was deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
       fi
   fi
   LOG_COUNTER=$(( LOG_COUNTER + 1 ))
 done
 if [ $LOG_COUNTER -lt $(( MAXLOGFILES + 1 )) ]
  then
     echo "   No cleanup necessary..."
     echo "   No cleanup necessary..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

### test if the script was successfully executed ###
### function_status
function_status ()
{
 if [ "$1" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   The script was successfully executed!"
     echo "   The script was successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"."
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   The script wasn\`t successfully executed!"
     echo "   The script wasn\`t successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

########################################################################################
### main									     ###
########################################################################################
### test if the script is started as root ###
if [ ! `whoami | grep root` ]
 then
    echo ""
    echo "   The script must be started as root \"sudo backup.sh\". Abort script!"
    echo ""
    exit 0
fi

### test if a startparameter is given ###
if [ -z "$1" -o "$1" = "help" -o "$1" != "start" ]
 then
    echo ""
    echo "   Usage: sudo backup.sh | start help"
    echo ""
    echo -e "$DESCRIPTION"
    echo "### Variables which should be configured ###"
    echo "SRC			# directory of the source (USB-stick or NFS-mount)"
    echo "DEST			# directory for the backup"
    echo ""
    exit 0
fi

### create log-file ###
function_createlog

### test if all directories are avialable ###
function_directory $SRC $DEST $COMPARE $COPY

### test if the copy.conf-file exists ###
function_fileexists $COPY_CONFIGFILE

### test if the compare.conf-file exists ###
function_fileexists $COMPARE_CONFIGFILE

### create backup of the copy-files ###"
echo "### Create backup of the copy-files ###"
echo "### Create backup of the copy-files ###" >> $LOGFILE
function_copy $COPY_CONFIGFILE $SRC $COPY $TYPE

### create backup of the compare-files ###
echo "### Create backup of the compare-files ###"
echo "### Create backup of the compare-files ###" >> $LOGFILE
function_copy $COMPARE_CONFIGFILE $SRC $COMPARE $TYPE

### cleanup the logfile-directory ###
echo "### Cleanup logfile-directory ###"
echo "### Cleanup logfile-directory ###" >> $LOGFILE
function_clean $TYPE

### test if the script was successfully executed ###
echo "### Backup complete ###"
echo "### Backup complete ###" >> $LOGFILE
function_status $ERROR_COUNTER

