#!/bin/sh

########################################################################################
# Description:	Backup-Script for the vip1710
#		Makes a backup of the hole stick without the backup-directory 
# Author:	Thomas Barth
# Date:		01.09.2011
# Version:	1.00
#
# Hint:		Feel free to modify the script
#		Feel also free to contact me
#		nessie_1980@hotmail.com
########################################################################################

DESCRIPTION="# This script makes a backup of of the hole stick without the backup-directory.\n"

### variables ###
SRC="/mnt/"						# directory of the destination (USB-Stick)	(changeable)
DEST="/mnt/backup/"					# directory for the backup			(changeable)
DISTRI=$DEST"distri/"					# directory for the log-file			(unchangeable)
LOG=$DEST"log/"						# directory for the log-file			(unchangeable)
SCRIPTPATH=$(cd -P "$( dirname "$0" )" && pwd )"/"	# directory of the script			(unchangeable)
TAR_CONFIGFILE=$SCRIPTPATH"tar_exclude.conf"		# copy-configfile				(unchangeable)
TYPE="complete_backup"					# backup					(unchangeable)
FILENAME=$TYPE".tar.gz"					# name of backupfile				(unchangeable)
DATELOGFILE=`date +"%d_%b_%Y_%H_%M_%S"`			#						(changeable)
LOGFILE=$LOG$TYPE"_"$DATELOGFILE".log"			# logfile					(unchangeable)
DATE=`date +"%d.%b.%Y at %H:%M:%S"`			#						(changeable)
MAXLOGFILES=5						# assign how many logfiles stored		(changeable)

# (unchangeable)
ERROR_COUNTER=0			# counter-variable
LOG_COUNTER=0			# counter-variable

########################################################################################
### functions									     ###
########################################################################################
### create log-file ###
### function_createlog
function_createlog ()
{
 echo "### Creating log-file ###"
 if [ -d "$LOG" ]
  then
     echo "   Directory \"$LOG\" exists..."
     echo ""
  else
     echo "   Directory \"$LOG\" doesn\`t exist!"
     mkdir -p $LOG 2> /dev/null
     if [ -d "$LOG" ]
       then
          echo "   Directory \"$LOG\" has been created..."
	  echo ""
       else
          echo "   Directory \"$LOG\" couldn\`t be created! Abort script!"
          echo ""
          exit 0
     fi
 fi
 touch $LOGFILE 2> /dev/null
 if [ "$?" != "0" ]
  then
     echo "   Log-file can´t be created! Abort script!"
     echo ""
     exit 0
 fi

 echo "### Log-file form "$DATE" ###" > $LOGFILE
 echo "   $LOGFILE"
 echo "   $LOGFILE" >> $LOGFILE
 echo ""
 echo "" >> $LOGFILE
}

### test if all directories are avialable ###
### function_directory directories (which should be checked)
function_directory ()
{
 for DIR in $@
  do
    echo "### Test if the directory \"$DIR\" exists ###" 
    echo "### Test if the directory \"$DIR\" exists ###" >> $LOGFILE
    if [ -d "$DIR" ]
     then
        echo "   Directory \"$DIR\" exists..."
        echo "   Directory \"$DIR\" exists..." >> $LOGFILE
        echo ""
        echo "" >> $LOGFILE
     else
        if [ "$DIR" != "$SRC" ]
         then
            echo "   Directory \"$DIR\" doesn\`t exist!"
            echo "   Directory \"$DIR\" doesn\`t exist!" >> $LOGFILE
	    mkdir -p $DIR 2> /dev/null
            if [ -d "$DIR" ]
             then
                echo "   Directory \"$DIR\" has been created..."
                echo "   Directory \"$DIR\" has been created..." >> $LOGFILE
	        echo ""
	        echo "" >> $LOGFILE
             else
                echo "   Directory \"$DIR\" couldn\`t be created! Abort script!"
                echo "   Directory \"$DIR\" couldn\`t be created! Abort script!" >> $LOGFILE
 	        echo ""
 	        echo "" >> $LOGFILE
                exit 0
            fi
        else
           echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!"
           echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
           exit 0
        fi
    fi
 done
}

### test if the copy.conf-file exists ###
### function_fileexists file
function_fileexists ()
{
 echo "### Test if the file \"$1\" exists ###" 
 echo "### Test if the file \"$1\" exists ###" >> $LOGFILE
 if [ -f "$1" ]
   then
      echo "   File \"$1\" exists..."
      echo "   File \"$1\" exists..." >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
   else
      echo "   File \"$1\" doesn\`t exists! Abort script!"
      echo "   File \"$1\" doesn\`t exists! Abort script!" >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
      exit 0
 fi
}

### packs all files of the stick without the backup-directory ###
### function_tar source destination
function_tar ()
{
 echo "### Create backup without the backup-directory ###"
 echo "### Create backup without the backup-directory ###" >> $LOGFILE

 ### extract the files or dircetories which should be exclueded from the tar ###
 EXCLUDEFILES=""
 EXCLUDE=""
 COUNTER=0

 for EXCLUDEFILES in `sed -e '/^[  ]*#/d' -e '/^$/d' $3`
  do
   if [ $COUNTER -eq 0 ]
    then
       EXCLUDE=$EXCLUDE$EXCLUDEFILES
    else
       EXCLUDE=$EXCLUDE"\|"$EXCLUDEFILES
   fi
   COUNTER=$(( $COUNTER + 1 ))
 done

 ### create tar-file
 echo "   tar -czf "$DISTRI"backup_complete.tar.gz" `ls $SRC | grep -v ${EXCLUDE}`
 echo "   tar -czf "$DISTRI"backup_complete.tar.gz" `ls $SRC | grep -v ${EXCLUDE}` >> $LOGFILE
 echo ""
 echo "" >> $LOGFILE
 echo "   This can take some time... So don\`t worry and take it easy :-)"
 echo "   This can take some time... So don\`t worry and take it easy :-)" >> $LOGFILE
 echo ""
 echo "" >> $LOGFILE

 cd $SRC
 tar -czf $DISTRI"backup_complete.tar.gz" `ls $SRC | grep -v ${EXCLUDE}`

 if [ "$?" = "0" ]
  then
     echo "   Tar was successfully created..."
     echo "   Tar was successfully created..." >> $LOGFILE
  else
     echo "   -=>FAILURE<=-"
     echo "   -=>FAILURE<=-" >> $LOGFILE
     echo "   Tar couldn\`t be created!"
     echo "   Tar couldn\`t be created!" >> $LOGFILE
     ERROR_COUNTER=$(( ERROR_COUNTER + 1 ))
 fi

 echo ""
 echo "" >> $LOGFILE
}

### cleanup log-directory ###
### function_clean search-string (in the filename)
function_clean ()
{
 for FILES in `ls -r $LOG | grep $1`
  do
   if [ $LOG_COUNTER -gt $(( MAXLOGFILES - 1 )) ]
    then
       echo "   Log-file: \"$FILES\" will be deleted!"
       echo "   Log-file: \"$FILES\" will be deleted!" >> $LOGFILE
       rm $LOG$FILES 2> /dev/null
       if [ -f "$LOG$FILEs" ]
        then
	   echo "   -=>FAILURE<=-"
	   echo "   -=>FAILURE<=-" >> $LOGFILE
           echo "   Log-file: \"$FILES\" wasn\`t deleted!"
           echo "   Log-file: \"$FILES\" wasn\`t deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
	   ERROR_COUNTER=$(( ERROR_COUNTER + 1 ))
        else
           echo "   Log-file: \"$FILES\" was deleted!"
           echo "   Log-file: \"$FILES\" was deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
       fi
   fi
   LOG_COUNTER=$(( LOG_COUNTER + 1 ))
 done
 if [ $LOG_COUNTER -lt $(( MAXLOGFILES + 1 )) ]
  then
     echo "   No cleanup necessary..."
     echo "   No cleanup necessary..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

### test if the script was successfully executed ###
### function_status
function_status ()
{
 if [ "$1" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   The script was successfully executed!"
     echo "   The script was successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"."
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   The script wasn\`t successfully executed!"
     echo "   The script wasn\`t successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

########################################################################################
### main									     ###
########################################################################################
### test if the script is started as root ###
if [ ! `whoami | grep root` ]
 then
    echo ""
    echo "   The script must be started as root \"sudo backup_complete.sh\". Abort script!"
    echo ""
    exit 0
fi

### test if a startparameter is given ###
if [ -z "$1" -o "$1" = "help" -o "$1" != "start" ]
 then
    echo ""
    echo "   Usage: sudo backup_complete.sh | start help"
    echo ""
    echo -e "$DESCRIPTION"
    echo "### Variables which should be configured ###"
    echo "SRC			# directory of the source (USB-stick or NFS-mount)"
    echo "DEST			# directory for the backup"
    echo ""
    exit 0
fi

### create log-file ###
function_createlog

### test if all directories are avialable ###
function_directory $SRC $DEST $DISTRI

### test if the copy.conf-file exists ###
function_fileexists $TAR_CONFIGFILE

### create backup of the copy-files ###"
function_tar $SRC $DEST $TAR_CONFIGFILE

### cleanup the logfile-directory ###
echo "### Cleanup logfile-directory ###"
echo "### Cleanup logfile-directory ###" >> $LOGFILE
function_clean $TYPE

### test if the script was successfully executed ###
echo "### Backup complete ###"
echo "### Backup complete ###" >> $LOGFILE
function_status $ERROR_COUNTER
