#!/bin/sh

########################################################################################
# Description:	Copy new distri to the vip1710 
#		Makes a backup of the config-files
#		Deletes the old distri and extract the new distri to the USB-stick
#		Does not delete the back-directory on the USB-stick
#		Restores the old config-files
# Author:	Thomas Barth
# Date:		01.09.2011
# Version:	1.00
#
# Hint:		Feel free to modify the script
#		Feel also free to contact me
#		nessie_1980@hotmail.com
########################################################################################

DESCRIPTION="# This script helps you to make a backup of specified files of your USB-stick or NFS-mount.
# In the first step the script checks if all directories are available. If they aren\`t available the script tries to create them.
# After that step the script backups the files which are specified in the files \"COPY_CONFIGFILE\" and \"COMPARE_CONFIGFILE\". 
# The file \"COPY_CONFIGFILE\" contains the files which will be backuped to the specified directory
# and later restored by the restore.sh-script to your USB-stick or NFS-mount. (1:1-Copy per CP-command)
# The file \"COMPARE_CONFIGFILE\" contains the files which will be backuped to the specified directory.
# Then the script deletes the USB-stick or NFS-mount, but doesn\`t delete the backup-directory on the USB-stick or NFS-mount.
# In the next step the script extracts the new distri-tar to the destination.
# Finally the script copies your old config-files of the file \"COPY_CONFIGFILE\" to the destination.
# The content of the files in the file \"COMPARE_CONFIGFILE\" are be compared with the content of the files of the destination (new distri).
# If the value of the backup-file is different to the value of the new file, the old values are stored
# to the new file. If a parameter is not available in the new file but is available in the backup-file,
# the parameter are restored from the back-file to the new file!!!\n"

### variables ###

SRC="/mnt/"						# directory of the source (USB-stick)		(changeable)
DEST="/mnt/backup/"					# directory for the backup			(changeable)
COMPARE=$DEST"compare/"					# directory for the compare-files		(unchangeable)
COPY=$DEST"copy/"					# directory for the copy-files			(unchangeable)
LOG=$DEST"log/"						# directory for the log-file			(unchangeable)
DISTRI=$DEST"new_distri/"					# directory for the log-file			(unchangeable)
SCRIPTPATH=$(cd -P "$( dirname "$0" )" && pwd )"/"	# directory of the script			(unchangeable)
COPY_CONFIGFILE=$SCRIPTPATH"copy.conf"			# copy-configfile				(unchangeable)
COMPARE_CONFIGFILE=$SCRIPTPATH"compare.conf"		# compare-configfile				(unchangeable)
ERASE_CONFIGFILE=$SCRIPTPATH"erase_exclude.conf"	# copy-configfile				(unchangeable)
TYPE="distri_update"					# distri_update					(unchangeable)
TYPE1="backup"						# backup					(unchangeable)
TYPE2="restore"						# restore					(unchangeable)
FILENAME=""						# name of the file which has been copied	(unchangeable)
DATELOGFILE=`date +"%d_%b_%Y_%H_%M_%S"`			#						(changeable)
LOGFILE=$LOG$TYPE"_"$DATELOGFILE".log"			# logfile					(unchangeable)
DATE=`date +"%d.%b.%Y at %H:%M:%S"`			#						(changeable)
MAXLOGFILES=5						# assign how many logfiles stored		(changeable)

# (unchangeable)
ERROR_COUNTER=0			# counter-variable
PARAMETER_COUNTER=0		# counter-variable
ERROR_COUNTER_BACKUP=0		# counter-variable for the backup
ERROR_COUNTER_DISTRI=0		# counter-variable for the distri
ERROR_COUNTER_CLEANUP=0		# counter-variable for the cleanup
ERROR_COUNTER_RESTORE=0		# counter-variable for the restore
ERASEFLAG=0			# flag for the erase-exection
LOG_COUNTER=0			# counter-variable

########################################################################################
### functions									     ###
########################################################################################
### create log-file ###
### function_createlog
function_createlog ()
{
 echo "### Creating log-file ###"
 if [ -d "$LOG" ]
  then
     echo "   Directory \"$LOG\" exists..."
     echo ""
  else
     echo "   Directory \"$LOG\" doesn\`t exist!"
     mkdir -p $LOG 2> /dev/null
     if [ -d "$LOG" ]
       then
          echo "   Directory \"$LOG\" has been created..."
	  echo ""
       else
          echo "   Directory \"$LOG\" couldn\`t be created! Abort script!"
          echo ""
          exit 0
     fi
 fi
 touch $LOGFILE 2> /dev/null
 if [ "$?" != "0" ]
  then
     echo "   Log-file can´t be created! Abort script!"
     echo ""
     exit 0
 fi

 echo "### Log-file form "$DATE" ###" > $LOGFILE
 echo "   $LOGFILE"
 echo "   $LOGFILE" >> $LOGFILE
 echo ""
 echo "" >> $LOGFILE
}

### test if all directories are avialable ###
### function_directory directories (which should be checked)
function_directory ()
{
 for DIR in $@
  do
   echo "### Test directory \"$DIR\" ###"
   echo "### Test directory \"$DIR\" ###" >> $LOGFILE
   if [ -d $DIR ]
    then
       echo "   Directory \"$DIR\" exists..."
       echo "   Directory \"$DIR\" exists..." >> $LOGFILE
       echo ""
       echo "" >> $LOGFILE
    else
       if [ "$DIR" != "$DEST" -a "$DIR" != "$DISTRI" ]
        then
           echo "   Directory \"$DIR\" doesn\`t exist!"
           echo "   Directory \"$DIR\" doesn\`t exist!" >> $LOGFILE
	   mkdir -p $DIR 2> /dev/null
           if [ -d $DIR ] 
            then
               echo "   Directory \"$DIR\" has been created..."
               echo "   Directory \"$DIR\" has been created..." >> $LOGFILE
 	       echo ""
 	       echo "" >> $LOGFILE
            else
               echo "   Directory \"$DIR\" couldn\`t be created! Abort script!"
               echo "   Directory \"$DIR\" couldn\`t be created! Abort script!" >> $LOGFILE
 	       echo ""
 	       echo "" >> $LOGFILE
               exit 0
           fi
       else
	  if [ "$DIR" = "$DEST" ]
   	   then
              echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!"
              echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!" >> $LOGFILE
              echo ""
              echo "" >> $LOGFILE
           else
              echo "   Directory \"$DIR\" (no Distri available) doesn\`t exist! Abort script!"
              echo "   Directory \"$DIR\" (no Distri available) doesn\`t exist! Abort script!" >> $LOGFILE
              echo ""
              echo "" >> $LOGFILE
	  fi
          exit 0
       fi 
   fi
 done
}

### determine distri-file ###
### function_distriexists
function_distriexists ()
{
 echo "### Determine distri-file ###" >> $LOGFILE
 for distrifiles in $DISTRI*
 do
   if [ `ls -A $DISTRI | wc -l` = "1" ]
    then
       echo "   Distri-file found: \"${distrifiles}\""
       echo "   Distri-file found: \"${distrifiles}\"" >> $LOGFILE
       echo ""
       echo "" >> $LOGFILE
       DISTRIFILE=${distrifiles}
    elif [ `ls -A $DISTRI | wc -l` = "0" ]
     then
        echo "   No distri-file available! Abort script!"
        echo "   No distri-file available! Abort script!" >> $LOGFILE
        echo ""
        echo "" >> $LOGFILE
        exit 0
    else
       echo "   Found to many files in the distri-directory! Abort script!"
       echo "   Found to many files in the distri-directory! Abort script!" >> $LOGFILE
       echo ""
       echo "" >> $LOGFILE
       exit 0
   fi
 done
}

### test if the copy.conf-file exists ###
### function_fileexists file
function_fileexists ()
{
 echo "### Test if the file \"$1\" exists ###" 
 echo "### Test if the file \"$1\" exists ###" >> $LOGFILE
 if [ -f "$1" ]
   then
      echo "   File \"$1\" exists..."
      echo "   File \"$1\" exists..." >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
   else
      echo "   File \"$1\" doesn\`t exists! Abort script!"
      echo "   File \"$1\" doesn\`t exists! Abort script!" >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
      exit 0
 fi
}

### copy files or directories from config-file or directory ###
### function_copy config-file source destination
function_copy ()
{
 if [ "$4" = "backup" ]
  then
     ### erase the content of the directory before the backup starts###
     echo "### Erase old backup-files or directories ###"
     echo "### Erase old backup-files or directories ###" >> $LOGFILE

     for FILES in `ls $3`
      do
        echo "   Erase: rm -r $3$FILES"
	echo "   Erase: rm -r $3$FILES" >> $LOGFILE
    	rm -r $3$FILES 2> /dev/null
    	if [ ! -e $3$FILES ]
     	 then
            echo "   File or directory \"$3$FILES\" was erased..."
            echo "   File or directory \"$3$FILES\" was erased..." >> $LOGFILE
	    echo ""
	    echo "" >> $LOGFILE
         else
	    echo "   -=>FAILURE<=-"
	    echo "   -=>FAILURE<=-" >> $LOGFILE
            echo "   File or directory \"$3$FILES\" wasn\`t erased!"
            echo "   File or directory \"$3$FILES\" wasn\`t erased!" >> $LOGFILE
            echo ""
            echo "" >> $LOGFILE
	    ERROR_COUNTER=$((ERROR_COUNTER + 1))
       fi
     done
 fi

 echo "### Copy new files or directories ###"
 echo "### Copy new files or directories ###" >> $LOGFILE

 ### determine the files or directoies which should be backuped ###
 for FILES in `sed -e '/^[  ]*#/d' -e '/^$/d' $1`
  do
    ### determine the name of the file or directory which was copied ###
    OIFS=$IFS
    IFS="/"
    for I in $FILES
     do
      FILENAME=$I
    done
    IFS=$OIFS

    ### test if the file or directory exists ###
    if [ "$4" = "backup" ]
     then
        if [ -f "$2$FILES" -o -d "$2$FILES" ]
         then
            echo "   File or directory found: \"$2$FILES\""
            echo "   File or directory found: \"$2$FILES\"" >> $LOGFILE
	    cp -R $2$FILES $3$FILENAME 2> /dev/null

	    ### test if the copy was successful ###
	    if [ -f "$3$FILENAME" -o -d "$3$FILENAME" ]
             then
                echo "   File or directory \"$2$FILES\" copied to \"$3$FILENAME\"..."
                echo "   File or directory \"$2$FILES\" copied to \"$3$FILENAME\"..." >> $LOGFILE
             else
	        echo "   -=>FAILURE<=-"
	        echo "   -=>FAILURE<=-" >> $LOGFILE
                echo "   File or directory \"$2$FILES\" wasn\`t copied to \"$3$FILENAME\"!"
                echo "   File or directory \"$2$FILES\" wasn\`t copied to \"$3$FILENAME\"!" >> $LOGFILE
	        ERROR_COUNTER=$((ERROR_COUNTER+1))
           fi
           echo ""
           echo "" >> $LOGFILE
        else
	   echo "   -=>FAILURE<=-"
	   echo "   -=>FAILURE<=-" >> $LOGFILE
           echo "   File or directory \"$2$FILES\" not found!"
           echo "   File or directory \"$2$FILES\" not found!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
           ERROR_COUNTER=$((ERROR_COUNTER+1))
        fi
    elif [ "$4" = "restore" ]
     then
        if [ -f "$2$FILENAME" -o -d "$2$FILENAME" ]
         then
            echo "   File or directory found: \"$2$FILENAME\""
            echo "   File or directory found: \"$2$FILENAME\"" >> $LOGFILE
	    cp -R $2$FILENAME $3$FILES 2> /dev/null

	    ### test if the copy was successful ###
	    if [ -f "$3$FILES" -o -d "$3$FILES" ]
             then
                echo "   File or directory \"$2$FILENAME\" copied to \"$3$FILES\"..."
                echo "   File or directory \"$2$FILENAME\" copied to \"$3$FILES\"..." >> $LOGFILE
             else
	        echo "   -=>FAILURE<=-"
	        echo "   -=>FAILURE<=-" >> $LOGFILE
                echo "   File or directory \"$2$FILENAME\" wasn\`t copied to \"$3$FILES\"!"
                echo "   File or directory \"$2$FILENAME\" wasn\`t copied to \"$3$FILES\"!" >> $LOGFILE
	        ERROR_COUNTER=$((ERROR_COUNTER+1))
            fi
            echo ""
            echo "" >> $LOGFILE
         else
	    echo "   -=>FAILURE<=-"
	    echo "   -=>FAILURE<=-" >> $LOGFILE
            echo "   File or directory \"$2$FILENAME\" not found!"
            echo "   File or directory \"$2$FILENAME\" not found!" >> $LOGFILE
            echo ""
            echo "" >> $LOGFILE
            ERROR_COUNTER=$((ERROR_COUNTER+1))
        fi
    fi
 done
}

### erase destination without the backup-directory and extract the new distri###
### function_distri exclude-file
function_distri ()
{
 ### extract the files or dircetories which should be exclueded from the tar ###
 EXCLUDEFILES=""
 EXCLUDE=""
 COUNTER=0

 for EXCLUDEFILES in `sed -e '/^[  ]*#/d' -e '/^$/d' $1`
  do
   if [ $COUNTER -eq 0 ]
    then
       EXCLUDE=$EXCLUDE$EXCLUDEFILES
    else
       EXCLUDE=$EXCLUDE"\|"$EXCLUDEFILES
   fi
   COUNTER=$(( $COUNTER + 1 ))
 done

 echo "### Erase USB-stick or NFS-mount without the backup-directory###"
 echo "### Erase USB-stick or NFS-mount without the backup-directory###" >> $LOGFILE
 if [ `ls $SRC | grep -v ${EXCLUDE} | wc -l` -gt 0 ]
  then
     for FILES in `ls $SRC | grep -v ${EXCLUDE}`
      do
        echo "   Erase: rm -r $FILES"
        echo "   Erase: rm -r $FILES" >> $LOGFILE
        rm -r $FILES 2> /dev/null
        if [ ! -e $FILES ]
         then
            echo "   File or directory \"$FILES\" was erased..."
            echo "   File or directory \"$FILES\" was erased..." >> $LOGFILE
	    echo ""
	    echo "" >> $LOGFILE
         else
	    echo "   -=>FAILURE<=-"
	    echo "   -=>FAILURE<=-" >> $LOGFILE
            echo "   File or directory \"$FILES\" wasn\`t erased!"
            echo "   File or directory \"$FILES\" wasn\`t erased!" >> $LOGFILE
            echo ""
            echo "" >> $LOGFILE
	    ERROR_COUNTER=$((ERROR_COUNTER + 1))
	    ERASEFLAG=1
       fi
     done
     for FILES in `ls $SRC | grep ${EXCLUDE}`
      do
        echo "   File or directory \"$FILES\" wasn\`t erased!"
        echo "   File or directory \"$FILES\" wasn\`t erased!" >> $LOGFILE
        echo ""
        echo "" >> $LOGFILE
     done
  else
     echo "   Destination is already deleted!"
     echo "   Destination is already deleted!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     ERASEFLAG=2
 fi

 ### test if the erase was successfully ###
 ### when the erase failed, the script will be canceled ###
 if [ $ERASEFLAG != 1 ]
  then
     if [ $ERASEFLAG != 2 ]
      then
         echo "   Destination was successfully deleted"
         echo "   Destination was successfully deleted" >> $LOGFILE
	 echo ""
	 echo "" >> $LOGFILE
     fi
     ### extract new distri to the stick ###
     echo "### Extract new distri to the USB-stick or NFS-mount ###"
     echo "### Extract new distri to the USB-stick or NFS-mount ###" >> $LOGFILE
     tar xzf $DISTRIFILE -C $SRC 2> /dev/null
     if [ "$?" = "0" ]
      then
         echo "   Extract was successfully"
         echo "   Extract was successfully" >> $LOGFILE
	 echo ""
	 echo "" >> $LOGFILE
      else
         echo "   Extract failed! Abort script!"
         echo "   Extract failed! Abort script!" >> $LOGFILE
	 echo ""
	 echo "" >> $LOGFILE
         exit 0
      fi
  else
     echo "The earse failed! Abort script!"
     echo "The earse failed! Abort script!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     exit 0
 fi
}

### restore values of the compare-files ###
### function_compare config-file USB-stick or NFS-mount
function_compare ()
for FILES in `sed -e '/^[  ]*#/d' -e '/^$/d' $1`
 do
  ### determine the name of the file which was copied ###
  OIFS=$IFS
  IFS="/"
  for I in $FILES
   do
    FILENAME=$I
  done
  IFS=$OIFS

  ### test if your own file exists ###
  if [ -f "$COMPARE$FILENAME" ]
   then
      echo "   Old file found: \"$COMPARE$FILENAME\""
      echo "   Old file found: \"$COMPARE$FILENAME\"" >> $LOGFILE

      ### test if the new distri-file exists ###
      if [ -f "$2$FILES" ]
       then
          echo "   Distri-file found: \"$2$FILES\"..."
          echo "   Distri-file found: \"$2$FILES\"..." >> $LOGFILE
	  echo ""
   	  echo "" >> $LOGFILE
 
          cp $COMPARE$FILENAME $COMPARE$FILENAME".tmp" 2> /dev/null

	  if [ "$?" != "0" ]
	   then
	      echo "   -=>FAILURE<=-"
	      echo "   -=>FAILURE<=-" >> $LOGFILE
	      echo "   Couldn\`t create Temp-file \"$COMPARE$FILENAME".tmp"\"! File skipped!"
	      echo "   Couldn\`t create Temp-file \"$COMPARE$FILENAME".tmp"\"! File skipped!" >> $LOGFILE
	      echo ""
   	      echo "" >> $LOGFILE
	      ERROR_COUNTER=$((ERROR_COUNTER+1))
	      break
	  fi

          ### starting to compare line per line###
	  echo "   The compare can take some time... So don\`t worry and take it easy :-)"
	  echo "   The compare can take some time... So don\`t worry and take it easy :-)" >> $LOGFILE
	  echo ""
   	  echo "" >> $LOGFILE

	  ### run throw the lines of the new file ###
	  while read SETTING_NEW
           do
	    ### reset flags ###
	    PARAMETER_FOUND=0

	    ### run throw the old settingvalues ###
	    OIFS=$IFS
  	    IFS="
"
	    for SETTING_OLD in `sed -e '' $COMPARE$FILENAME".tmp"`
             do
	      ### split row into the parameter and the value ###
	      PARA_NEW=$(echo $SETTING_NEW | cut -d"=" -f1)
	      VAL_NEW=$(echo $SETTING_NEW | cut -d"=" -f2)
	      PARA_OLD=$(echo $SETTING_OLD | cut -d"=" -f1)
	      VAL_OLD=$(echo $SETTING_OLD | cut -d"=" -f2)

	      ### compare the parameter-names ###
	      if [ "$PARA_NEW" = "$PARA_OLD" ]
	       then
		  ### compare the values ###
		  if [ "$VAL_NEW" = "$VAL_OLD" ]
 		   then
   		      #echo "   Parameter found in both files! ### New-File: "$SETTING_NEW" ### Old-File: "$SETTING_OLD""
   		      #echo "   Parameter found in both files! ### New-File: "$SETTING_NEW" ### Old-File: "$SETTING_OLD"" >> $LOGFILE
  		      #echo "   Values are equal! Write new value..."
  		      #echo "   Values are equal! Write new value..." >> $LOGFILE
		      #echo ""
		      #echo "" >> $LOGFILE
		      if [ $PARAMETER_COUNTER -lt 1 ]
		       then
			  echo $SETTING_NEW > $2$FILES".tmp"
		       else
		          echo $SETTING_NEW >> $2$FILES".tmp"
		      fi
		      PARAMETER_COUNTER=$(( PARAMETER_COUNTER + 1 ))
		   else
         	      echo "   Parameter found in both files! ### New-File: "$SETTING_NEW" ### Old-File: "$SETTING_OLD""
         	      echo "   Parameter found in both files! ### New-File: "$SETTING_NEW" ### Old-File: "$SETTING_OLD"" >> $LOGFILE
		      echo "   Values are different! Write old value..."
		      echo "   Values are different! Write old value..." >> $LOGFILE
		      echo ""
		      echo "" >> $LOGFILE
		      if [ $PARAMETER_COUNTER -lt 1 ]
		       then
	                  echo $SETTING_OLD > $2$FILES".tmp"
		       else
		          echo $SETTING_OLD >> $2$FILES".tmp"
		      fi
		      PARAMETER_COUNTER=$(( PARAMETER_COUNTER + 1 ))
		  fi
		  sed -e ''`cat "$COMPARE$FILENAME"".tmp" | grep -n "^$SETTING_OLD" | cut -d ":" -f 1`d'' $COMPARE$FILENAME".tmp" > $COMPARE$FILENAME".tmp2"
		  mv $COMPARE$FILENAME".tmp2" $COMPARE$FILENAME".tmp" 2> /dev/null
		  if [ "$?" != "0" ]
		   then
	 	      echo "   -=>FAILURE<=-"
		      echo "   -=>FAILURE<=-" >> $LOGFILE
		      echo "   Temp-file \"$COMPARE$FILENAME".tmp"\" can\`t be moved"
		      echo "   Temp-file \"$COMPARE$FILENAME".tmp"\" can\`t be moved" >> $LOGFILE
		      ERROR_COUNTER=$((ERROR_COUNTER+1))
		      break
	  	  fi

		  PARAMETER_FOUND=1
		  break
	      fi
            done

	    ### if the parameter is not found in the old file, write new parameter ###
	    if [ "$PARAMETER_FOUND" = "0" ]
             then
       		echo "   Parameter not found in the old file! ### New-File: "$SETTING_NEW
       		echo "   Parameter not found in the old file! ### New-File: "$SETTING_NEW >> $LOGFILE
		echo "   Write parameter from the new file..."
		echo "   Write parameter from the new file..." >> $LOGFILE
     	 	echo ""
     	 	echo "" >> $LOGFILE
		if [ $PARAMETER_COUNTER -lt 1 ]
		 then
	            echo $SETTING_NEW > $2$FILES".tmp"
		 else
		    echo $SETTING_NEW >> $2$FILES".tmp"
		fi
		PARAMETER_COUNTER=$((PARAMETER_COUNTER + 1 ))
            fi
          done < $2$FILES

	  IFS=$OIFS

	  ### if there are still the parameters in the old file, write them in the new-file ###
	  for SETTING_OLD in `sed -e '' $COMPARE$FILENAME".tmp"`
           do
            echo "   Parameter not found in new file! ### Old-File: "$SETTING_OLD""
            echo "   Parameter not found in new file! ### Old-File: "$SETTING_OLD"" >> $LOGFILE
	    echo "   Write old value..."
	    echo "   Write old value..." >> $LOGFILE
	    echo ""
	    echo "" >> $LOGFILE
	    if [ $PARAMETER_COUNTER -lt 1 ]
	     then
	        echo $SETTING_OLD > $2$FILES".tmp"
	     else
	        echo $SETTING_OLD >> $2$FILES".tmp"
	    fi
	  PARAMETER_COUNTER=$(( PARAMETER_COUNTER + 1 ))
	  done

	  ### move temp-file to new-file ###
	  echo "### Move temp-file to new-file ###"
	  echo "### Move temp-file to new-file ###" >> $LOGFILE
          mv $2$FILES".tmp" $2$FILES 2> /dev/null
	  if [ "$?" != "0" ]
	   then
	      echo "   -=>FAILURE<=-"
	      echo "   -=>FAILURE<=-" >> $LOGFILE
	      echo "   Can\`t move temp-file \"$2$FILES".tmp"\" to new-file \"$2$FILES\"!"
	      echo "   Can\`t Move temp-file \"$2$FILES".tmp"\" to new-file \"$2$FILES\"!" >> $LOGFILE
   	      echo ""
   	      echo "" >> $LOGFILE
	      ERROR_COUNTER=$((ERROR_COUNTER+1))
	   else
	      echo "   Move temp-file \"$2$FILES".tmp"\" to new-file \"$2$FILES\"!"
	      echo "   Move temp-file \"$2$FILES".tmp"\" to new-file \"$2$FILES\"!" >> $LOGFILE
   	      echo ""
   	      echo "" >> $LOGFILE
	  fi

	  ### remove temp-file ###
	  echo "### Remove temp-file ###"
	  echo "### Remove temp-file ###" >> $LOGFILE
          rm $COMPARE$FILENAME".tmp" 2> /dev/null
	  if [ "$?" != "0" ]
	   then
	      echo "   -=>FAILURE<=-"
	      echo "   -=>FAILURE<=-" >> $LOGFILE
	      echo "   Can\`t remove temp-file \"$COMPARE$FILENAME".tmp"\"!"
	      echo "   Can\`t remove temp-file \"$COMPARE$FILENAME".tmp"\"!" >> $LOGFILE
   	      echo ""
   	      echo "" >> $LOGFILE
	      ERROR_COUNTER=$((ERROR_COUNTER+1))
	   else
	      echo "   Remove temp-file \"$COMPARE$FILENAME".tmp"\"!"
	      echo "   Remove temp-file \"$COMPARE$FILENAME".tmp"\"!" >> $LOGFILE
   	      echo ""
   	      echo "" >> $LOGFILE
	  fi
       else
          echo "   Distri-file \"$2$FILES\" not found! File wasn\`t compared and restored!"
          echo "   Distri-file \"$2$FILES\" not found! File wasn\`t compared and restored!" >> $LOGFILE
	  echo ""
	  echo "" >> $LOGFILE
      fi
   else
      echo "   Old file \"$COMPARE$FILENAME\" not found! File wasn\`t compared and restored!"
      echo "   Old file \"$COMPARE$FILENAME\" not found! File wasn\`t compared and restored" >> $LOGFILE
      echo ""
      echo "" >> $LOGFILE
  fi
done

### cleanup log-directory ###
### function_clean search-string (in the filename)
function_clean ()
{
 for FILES in `ls -r $LOG | grep $1`
  do
   if [ $LOG_COUNTER -gt $(( MAXLOGFILES - 1 )) ]
    then
       echo "   Delete Log-file \"$FILES\"..."
       echo "   Delete Log-file \"$FILES\"..." >> $LOGFILE
       rm $LOG$FILES 2> /dev/null
       if [ -f "$LOG$FILEs" ]
        then
	   echo "   -=>FAILURE<=-"
	   echo "   -=>FAILURE<=-" >> $LOGFILE
           echo "   Log-file \"$FILES\" wasn\`t deleted!"
           echo "   Log-file \"$FILES\" wasn\`t deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
	   ERROR_COUNTER=$(( ERROR_COUNTER + 1 ))
        else
           echo "   Log-file \"$FILES\" was deleted!"
           echo "   Log-file \"$FILES\" was deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
       fi
   fi
   LOG_COUNTER=$(( LOG_COUNTER + 1 ))
 done
 if [ $LOG_COUNTER -lt $(( MAXLOGFILES + 1 )) ]
  then
     echo "   No cleanup necessary..."
     echo "   No cleanup necessary..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

### test if the script was successfully executed ###
### function_status errors
function_status ()
{
 if [ "$1" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo "   The script was successfully executed!"
     echo "   The script was successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"."
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"." >> $LOGFILE
     echo "   The script wasn\`t successfully executed!"
     echo "   The script wasn\`t successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

### summary ###
### function_summary
function_summary ()
{
 echo "#######################################"
 echo "#######################################" >> $LOGFILE
 echo "###             SUMMARY             ###"
 echo "###             SUMMARY             ###" >> $LOGFILE
 echo "#######################################"
 echo "#######################################" >> $LOGFILE
 echo ""
 echo "### Backup-Summary ###"
 echo "### Backup-Summary ###" >> $LOGFILE
 echo ""  >> $LOGFILE
 if [ "$ERROR_COUNTER_BACKUP" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $ERROR_COUNTER_BACKUP failure occurred..."
     echo "   $ERROR_COUNTER_BACKUP failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi

 echo "### Distri-Summary ###"
 echo "### Distri-Summary ###" >> $LOGFILE
 if [ "$ERROR_COUNTER_DISTRI" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $ERROR_COUNTER_DISTRI failure occurred..."
     echo "   $ERROR_COUNTER_DISTRI failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi

 echo "### Restore-Summary ###"
 echo "### Restore-Summary ###" >> $LOGFILE
 if [ "$ERROR_COUNTER_RESTORE" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $ERROR_COUNTER_RESTORE failure occurred..."
     echo "   $ERROR_COUNTER_RESTORE failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi

 echo "### Cleanup-Summary ###"
 echo "### Cleanup-Summary ###" >> $LOGFILE
 if [ "$ERROR_COUNTER_CLEANUP" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $ERROR_COUNTER_CLEANUP failure occurred..."
     echo "   $ERROR_COUNTER_CLEANUP failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi

echo "### Final-Summary ###"
echo "### Final-Summary ###" >> $LOGFILE
if [ "$ERROR_COUNTER_BACKUP" = "0" -a "$ERROR_COUNTER_DISTRI" = "0" -a "$ERROR_COUNTER_RESTORE" = "0" -a "$ERROR_COUNTER_CLEANUP" = "0" ]
 then
    echo "   No failure occurred..."
    echo "   No failure occurred..." >> $LOGFILE
    echo ""
    echo "" >> $LOGFILE
    echo "   The script was successfully executed!"
    echo "   The script was successfully executed!" >> $LOGFILE
    echo ""
    echo "" >> $LOGFILE
 else
    echo "   `expr $ERROR_COUNTER_BACKUP  + $ERROR_COUNTER_DISTRI + $ERROR_COUNTER_RESTORE + $ERROR_COUNTER_CLEANUP` failure occurred... Watch the screen or log-file \"$LOGFILE\"."
    echo "   `expr $ERROR_COUNTER_BACKUP  + $ERROR_COUNTER_DISTRI + $ERROR_COUNTER_RESTORE + $ERROR_COUNTER_CLEANUP` failure occurred... Watch the screen or log-file \"$LOGFILE\"." >> $LOGFILE
    echo ""
    echo "" >> $LOGFILE
    echo "   The script wasn\`t successfully executed!"
    echo "   The script wasn\`t successfully executed!" >> $LOGFILE
    echo ""
    echo "" >> $LOGFILE
fi
}

########################################################################################
### main									     ###
########################################################################################
### test if the script is started as root ###
if [ ! `whoami | grep root` ]
 then
    echo ""
    echo "   The script must be started as root \"sudo backup.sh\". Abort script!"
    echo ""
    exit 0
fi

### test if a startparameter is given ###
if [ -z "$1" -o "$1" = "help" -o "$1" != "start" ]
 then
    echo ""
    echo "   Usage: sudo distri_update.sh | start help"
    echo ""
    echo -e "$DESCRIPTION"
    echo "### Variables which should be configured ###"
    echo "SRC			# directory of the source (USB-stick or NFS-mount)"
    echo "DEST			# directory for the backup"
    echo ""
    exit 0
fi

### create log-file ###
function_createlog

### test directories ###
function_directory $SRC $DEST $COPY $COMPARE $DISTRI

### determine distri-file ###
function_distriexists

### test if the copy.conf-file exists ###
function_fileexists $COPY_CONFIGFILE

### test if the compare.conf-file exists ###
function_fileexists $COMPARE_CONFIGFILE

### test if the erase_exclude.conf-file exists ###
function_fileexists $ERASE_CONFIGFILE

ERROR_COUNTER=0
### create backup of the copy-files ###"
echo "### Create backup of the copy-files ###"
echo "### Create backup of the copy-files ###" >> $LOGFILE
function_copy $COPY_CONFIGFILE $SRC $COPY $TYPE1

### create backup of the compare-files ###
echo "### Create backup of the compare-files ###"
echo "### Create backup of the compare-files ###" >> $LOGFILE
function_copy $COMPARE_CONFIGFILE $SRC $COMPARE $TYPE1

ERROR_COUNTER_BACKUP=$(( $ERROR_COUNTER ))
### test if the script was successfully executed ###
echo "### Backup complete ###"
echo "### Backup complete ###" >> $LOGFILE
function_status $ERROR_COUNTER_BACKUP
ERROR_COUNTER=0

### erase destination without the backup-directory and extract the new distri###
function_distri $ERASE_CONFIGFILE

ERROR_COUNTER_DISTRI=$(( $ERROR_COUNTER ))
### test if the script was successfully executed ###
echo "### Distri-update complete ###"
echo "### Distri-update complete ###" >> $LOGFILE
function_status $ERROR_COUNTER_DISTRI
ERROR_COUNTER=0

### restore backup of the copy-files ###
echo "### Restore backup of the copy-files ###"
echo "### Restore backup of the copy-files ###" >> $LOGFILE
function_copy $COPY_CONFIGFILE $COPY $SRC $TYPE2

### restore values of the compare-files ###
echo "### Restore values of the compare-files ###"
echo "### Restore values of the compare-files ###" >> $LOGFILE
function_compare $COMPARE_CONFIGFILE $SRC

ERROR_COUNTER_RESTORE=$(( $ERROR_COUNTER ))
### test if the restore was successfully executed ###
echo "### Restore complete ###"
echo "### Restore complete ###" >> $LOGFILE
function_status $ERROR_COUNTER_RESTORE
ERROR_COUNTER=0

ERROR_COUNTER_CLEANUP=$(( $ERROR_COUNTER ))
### cleanup the logfile-directory ###
echo "### Cleanup logfile-directory ###"
echo "### Cleanup logfile-directory ###" >> $LOGFILE
function_clean $TYPE
ERROR_COUNTER=0

### summary ###
function_summary


