#!/bin/sh

########################################################################################
# Description:	Restore-Script for the vip1710
#		Makes a restore of a tar-file the backup-directory 
# Author:	Thomas Barth
# Date:		01.09.2011
# Version:	1.00
#
# Hint:		Feel free to modify the script
#		Feel also free to contact me
#		nessie_1980@hotmail.com
########################################################################################

DESCRIPTION="# This script makes a restore of a tar-file the backup-directory.\n"

### variables ###
SRC="/mnt/"						# directory of the destination (USB-Stick)	(changeable)
BACKUPSRC=$SRC"backup"					# backup-directory on the source (USB-stick)	(changeable) 
DEST="/mnt/backup/"					# directory for the backup			(changeable)
DISTRI=$DEST"distri/"					# directory for the log-file			(unchangeable)
LOG=$DEST"log/"						# directory for the log-file			(unchangeable)
SCRIPTPATH=$(cd -P "$( dirname "$0" )" && pwd )"/"	# directory of the script			(unchangeable)
TYPE="complete_restore"					# backup					(unchangeable)
FILENAME=""						# name of the file which has been copied	(unchangeable)
DATELOGFILE=`date +"%d_%b_%Y_%H_%M_%S"`			#						(changeable)
LOGFILE=$LOG$TYPE"_"$DATELOGFILE".log"			# logfile					(unchangeable)
DATE=`date +"%d.%b.%Y at %H:%M:%S"`			#						(changeable)
MAXLOGFILES=5						# assign how many logfiles stored		(changeable)

# (unchangeable)
ERROR_COUNTER=0			# counter-variable
LOG_COUNTER=0			# counter-variable
ERASEFLAG=0			# flag for the erase-exection

########################################################################################
### functions									     ###
########################################################################################
### create log-file ###
### function_createlog
function_createlog ()
{
 echo "### Creating log-file ###"
 if [ -d "$LOG" ]
  then
     echo "   Directory \"$LOG\" exists..."
     echo ""
  else
     echo "   Directory \"$LOG\" doesn\`t exist!"
     mkdir -p $LOG 2> /dev/null
     if [ -d "$LOG" ]
       then
          echo "   Directory \"$LOG\" has been created..."
	  echo ""
       else
          echo "   Directory \"$LOG\" couldn\`t be created! Abort script!"
          echo ""
          exit 0
     fi
 fi
 touch $LOGFILE 2> /dev/null
 if [ "$?" != "0" ]
  then
     echo "   Log-file can´t be created! Abort script!"
     echo ""
     exit 0
 fi

 echo "### Log-file form "$DATE" ###" > $LOGFILE
 echo "   $LOGFILE"
 echo "   $LOGFILE" >> $LOGFILE
 echo ""
 echo "" >> $LOGFILE
}

### test if all directories are avialable ###
### function_directory directories (which should be checked)
function_directory ()
{
 for DIR in $@
  do
    echo "### Test if the directory \"$DIR\" exists ###" 
    echo "### Test if the directory \"$DIR\" exists ###" >> $LOGFILE
    if [ -d "$DIR" ]
     then
        echo "   Directory \"$DIR\" exists..."
        echo "   Directory \"$DIR\" exists..." >> $LOGFILE
        echo ""
        echo "" >> $LOGFILE
     else
        if [ "$DIR" != "$SRC" ]
         then
            echo "   Directory \"$DIR\" doesn\`t exist!"
            echo "   Directory \"$DIR\" doesn\`t exist!" >> $LOGFILE
	    mkdir -p $DIR 2> /dev/null
            if [ -d "$DIR" ]
             then
                echo "   Directory \"$DIR\" has been created..."
                echo "   Directory \"$DIR\" has been created..." >> $LOGFILE
	        echo ""
	        echo "" >> $LOGFILE
             else
                echo "   Directory \"$DIR\" couldn\`t be created! Abort script!"
                echo "   Directory \"$DIR\" couldn\`t be created! Abort script!" >> $LOGFILE
 	        echo ""
 	        echo "" >> $LOGFILE
                exit 0
            fi
        else
           echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!"
           echo "   Directory \"$DIR\" (Source: USB-Stick or NFS-mount) doesn\`t exist! Abort script!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
           exit 0
        fi
    fi
 done
}

### determine restore-file ###
### function_restoreexists
function_distriexists ()
{
 echo "### Determine restore-file ###" >> $LOGFILE
 for restorefiles in $DISTRI*$DISTRIEXT
 do
   if [ `ls -A $DISTRI | wc -l` = "1" ]
    then
       echo "   Restore-file found: \"$restorefiles\""
       echo "   Restore-file found: \"$restorefiles\"" >> $LOGFILE
       echo ""
       echo "" >> $LOGFILE
       DISTRIFILE=$restorefiles
    elif [ `ls -A $DISTRI | wc -l` = "0" ]
     then
        echo "   No restore-file available! Abort script!"
        echo "   No restore-file available! Abort script!" >> $LOGFILE
        echo ""
        echo "" >> $LOGFILE
        exit 0
    else
       echo "   Found to many files in the distri-directory! Abort script!"
       echo "   Found to many files in the distri-directory! Abort script!" >> $LOGFILE
       echo ""
       echo "" >> $LOGFILE
       exit 0
   fi
 done
}

### erase destination without the backup-directory and extract the restore-file###
### function_distri
function_distri ()
{
 echo "### Erase USB-stick or NFS-mount without the backup-directory###"
 echo "### Erase USB-stick or NFS-mount without the backup-directory###" >> $LOGFILE
 if [ `ls $SRC | wc -l` -gt 0 ]
  then
     for FILES in $SRC*
      do
       if [ "$FILES" != "$BACKUPSRC" ]
        then
           echo "   Erase: rm -r $FILES"
           echo "   Erase: rm -r $FILES" >> $LOGFILE
           rm -r $FILES 2> /dev/null
           if [ ! -e $FILES ]
            then
               echo "   File / directory \"$FILES\" was erased..."
               echo "   File / directory \"$FILES\" was erased..." >> $LOGFILE
	       echo ""
	       echo "" >> $LOGFILE
            else
	       echo "   -=>FAILURE<=-"
	       echo "   -=>FAILURE<=-" >> $LOGFILE
               echo "   File / directory \"$FILES\" wasn\`t erased!"
               echo "   File / directory \"$FILES\" wasn\`t erased!" >> $LOGFILE
               echo ""
               echo "" >> $LOGFILE
	       ERROR_COUNTER=$((ERROR_COUNTER + 1))
	       ERASEFLAG=1
           fi
        else
	   echo "   Erase: -r $FILES"
	   echo "   Erase: -r $FILES" >> $LOGFILE
           echo "   File / directory \"$FILES\" wasn\`t erased!"
           echo "   File / directory \"$FILES\" wasn\`t erased!" >> $LOGFILE
	   echo ""
	   echo "" >> $LOGFILE
       fi
   done
  else
     echo "   Destination is already deleted!"
     echo "   Destination is already deleted!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     ERASEFLAG=2
 fi

 ### test if the erase was successfully ###
 ### when the erase failed, the script will be canceled ###
 if [ $ERASEFLAG != 1 ]
  then
     if [ $ERASEFLAG != 2 ]
      then
         echo "   Destination was successfully deleted"
         echo "   Destination was successfully deleted" >> $LOGFILE
	 echo ""
	 echo "" >> $LOGFILE
     fi
     ### extract new distri to the stick ###
     echo "### Extract restore-file to the USB-stick or NFS-mount ###"
     echo "### Extract restore-file to the USB-stick or NFS-mount ###" >> $LOGFILE
     echo "   tar -xzf "$DISTRIFILE" -C "$SRC
     echo "   tar -xzf "$DISTRIFILE" -C "$SRC >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   This can take some time... So don\`t worry and take it easy :-)"
     echo "   This can take some time... So don\`t worry and take it easy :-)" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE

     tar xzf $DISTRIFILE -C $SRC 2> /dev/null
     if [ "$?" = "0" ]
      then
         echo "   Extract was successfully"
         echo "   Extract was successfully" >> $LOGFILE
	 echo ""
	 echo "" >> $LOGFILE
      else
         echo "   Extract failed! Abort script!"
         echo "   Extract failed! Abort script!" >> $LOGFILE
	 echo ""
	 echo "" >> $LOGFILE
         exit 0
      fi
  else
     echo "The earse failed! Abort script!"
     echo "The earse failed! Abort script!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     exit 0
 fi
}

### cleanup log-directory ###
### function_clean search-string (in the filename)
function_clean ()
{
 for FILES in `ls -r $LOG | grep $1`
  do
   if [ $LOG_COUNTER -gt $(( MAXLOGFILES - 1 )) ]
    then
       echo "   Log-file: \"$FILES\" will be deleted!"
       echo "   Log-file: \"$FILES\" will be deleted!" >> $LOGFILE
       rm $LOG$FILES 2> /dev/null
       if [ -f "$LOG$FILEs" ]
        then
	   echo "   -=>FAILURE<=-"
	   echo "   -=>FAILURE<=-" >> $LOGFILE
           echo "   Log-file: \"$FILES\" wasn\`t deleted!"
           echo "   Log-file: \"$FILES\" wasn\`t deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
	   ERROR_COUNTER=$(( ERROR_COUNTER + 1 ))
        else
           echo "   Log-file: \"$FILES\" was deleted!"
           echo "   Log-file: \"$FILES\" was deleted!" >> $LOGFILE
           echo ""
           echo "" >> $LOGFILE
       fi
   fi
   LOG_COUNTER=$(( LOG_COUNTER + 1 ))
 done
 if [ $LOG_COUNTER -lt $(( MAXLOGFILES + 1 )) ]
  then
     echo "   No cleanup necessary..."
     echo "   No cleanup necessary..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

### test if the script was successfully executed ###
### function_status
function_status ()
{
 if [ "$1" = "0" ]
  then
     echo "   No failure occurred..."
     echo "   No failure occurred..." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   The script was successfully executed!"
     echo "   The script was successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
  else
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"."
     echo "   $1 failure occurred... Watch the screen or log-file \"$LOGFILE\"." >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
     echo "   The script wasn\`t successfully executed!"
     echo "   The script wasn\`t successfully executed!" >> $LOGFILE
     echo ""
     echo "" >> $LOGFILE
 fi
}

########################################################################################
### main									     ###
########################################################################################
### test if the script is started as root ###
if [ ! `whoami | grep root` ]
 then
    echo ""
    echo "   The script must be started as root \"sudo restore_complete.sh\". Abort script!"
    echo ""
    exit 0
fi

### test if a startparameter is given ###
if [ -z "$1" -o "$1" = "help" -o "$1" != "start" ]
 then
    echo ""
    echo "   Usage: sudo restore_complete.sh | start help"
    echo ""
    echo -e "$DESCRIPTION"
    echo "### Variables which should be configured ###"
    echo "SRC			# directory of the source (USB-stick or NFS-mount)"
    echo "DEST			# directory for the backup"
    echo ""
    exit 0
fi

### create log-file ###
function_createlog

### test if all directories are avialable ###
function_directory $SRC $DEST $DISTRI

### determine distri-file ###
function_distriexists

### erase destination without the backup-directory and extract the restore-file###
function_distri

### cleanup the logfile-directory ###
echo "### Cleanup logfile-directory ###"
echo "### Cleanup logfile-directory ###" >> $LOGFILE
function_clean $TYPE

### test if the script was successfully executed ###
echo "### Restore complete ###"
echo "### Restore complete ###" >> $LOGFILE
function_status $ERROR_COUNTER
